<?php

namespace Drupal\monarch_symlink\Controller;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Entity\EntityFormBuilderInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\HttpKernel\HttpKernelInterface;
use Drupal\Core\Controller\ControllerResolverInterface;
use Drupal\Core\Entity\Controller\EntityViewController;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\EventDispatcher\LegacyEventDispatcherProxy;
use Symfony\Component\HttpKernel\Controller\ArgumentResolverInterface;

/**
 * Defines a controller to render a symlink.
 */
class SymlinkControllerBase extends EntityViewController {

  /**
   * The http kernel service.
   *
   * @var \Symfony\Component\HttpKernel\HttpKernelInterface
   */
  protected $kernel;

  /**
   * The event dispatcher service.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $dispatcher;

  /**
   * The controller resolver service.
   *
   * @var \Drupal\Core\Controller\ControllerResolverInterface
   */
  protected $controllerResolver;

  /**
   * The the argument resolver service.
   *
   * @var \Symfony\Component\HttpKernel\Controller\ArgumentResolverInterface
   */
  protected $argumentResolver;

  /**
   * The entity form builder.
   *
   * @var \Drupal\Core\Entity\EntityFormBuilderInterface
   */
  protected $entityFormBuilder;

  /**
   * Creates a SymlinkController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Symfony\Component\HttpKernel\HttpKernelInterface $kernel
   *   The http kernel service.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $dispatcher
   *   The event dispatcher service.
   * @param \Drupal\Core\Controller\ControllerResolverInterface
   *   The controller resolver service.
   * @param \Symfony\Component\HttpKernel\Controller\ArgumentResolverInterface
   *   The the argument resolver service.
   * @param \Drupal\Core\Entity\EntityFormBuilderInterface $entity_form_builder
   *   The entity form builder.
   */
  public function __construct(
      EntityTypeManagerInterface $entity_type_manager,
      RendererInterface $renderer,
      HttpKernelInterface $kernel,
      EventDispatcherInterface $dispatcher,
      ControllerResolverInterface $controller_resolver,
      ArgumentResolverInterface $argument_resolver,
      EntityFormBuilderInterface $entity_form_builder) {

    parent::__construct($entity_type_manager, $renderer);

    $this->kernel = $kernel;
    $this->dispatcher = $dispatcher;
    $this->controllerResolver = $controller_resolver;
    $this->argumentResolver = $argument_resolver;
    $this->entityFormBuilder = $entity_form_builder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $dispatcher = $container->get('event_dispatcher');
    $dispatcher = LegacyEventDispatcherProxy::decorate($dispatcher);

    return new static(
      $container->get('entity_type.manager'),
      $container->get('renderer'),
      $container->get('http_kernel.basic'),
      $dispatcher,
      $container->get('controller_resolver'),
      $container->get('http_kernel.controller.argument_resolver'),
      $container->get('entity.form_builder')
    );
  }

  /**
   * Retrieves the entity type manager.
   *
   * @return \Drupal\Core\Entity\EntityTypeManagerInterface
   *   The entity type manager.
   */
  protected function entityTypeManager() {
    return $this->entityTypeManager;
  }

  /**
   * Retrieves the entity form builder.
   *
   * @return \Drupal\Core\Entity\EntityFormBuilderInterface
   *   The entity form builder.
   */
  protected function entityFormBuilder() {
    return $this->entityFormBuilder;
  }

}
